#!/bin/bash

# This helper script will handle deploying an AWS Lambda.
#
# /scripts/deploy-lambda.sh
#

set -e

source /scripts/utils.sh
source /scripts/terraform.sh

check_access ${appsNamespace} configmaps

tfAdditionalVarsFile=${tfRoot}/additional_settings.auto.tfvars

echo "iam_policy_prefix = \"${externalNamingPrefix}\"" >> ${tfAdditionalVarsFile}

if [ ! -z "${infraEnvMount}" ]; then
  infraVars=$(echo ${infraEnvMount} | jq -r '.[].envName')
  cprint "Starting configmap mounting var setup.."
  for var in ${infraVars}; do
    key=${var}
    # Find the mounted configmap variable
    value=$(printenv "${var}")
    # Create a template for assembling the map
    varTemplate='"%s":"%s",'
    jsonString=$(printf "${varTemplate}" "${key}" "${value}")
    # Append the template via the loop
    FILES+=${jsonString}
  done
  # Add brackets around at the end.
  output=$(echo {${FILES}})
  echo "infra_mount = ${output}" >> ${tfAdditionalVarsFile}
fi

if [ ! -z "${ecVar}" ]; then
  cprint "Starting EC var setup.."
  ecTemplate='{"%s":"%s", "%s":"%s"}'
  jsonString=$(printf "${ecTemplate}" \
  "${ecTokenEnvName}" "${ecToken}" \
  "${ecEndpointEnvName}" "${ecEndpoint}")
  echo "environment = ${jsonString}" >> ${tfAdditionalVarsFile}
elif [ ! -z "${pgVar}" ]; then
  cprint "Starting PG var setup.."
  pgTemplate='{ "%s":"%s","%s":"%s","%s":"%s","%s":"%s" }'
  jsonString=$(printf "${pgTemplate}" \
  "${pgUsernameEnvName}" "${pgUsername}" \
  "${pgPasswordEnvName}" "${pgPassword}" \
  "${pgEndpointEnvName}" "${pgEndpoint}" \
  "${pgReaderEndpointEnvName}" "${pgReaderEndpoint}")
  echo "environment = ${jsonString}" >> ${tfAdditionalVarsFile}
fi

if [ ! -z "${extraVar}" ]; then
  echo "extra_environment = ${vars}" >> ${tfAdditionalVarsFile}
fi

if [ ! -z "${triggerArn}" ]; then
  echo "event_source_arn = \"${triggerArn}\"" >> ${tfAdditionalVarsFile}
fi

# Adds the dlq arn if dlq settings are enabled
if [ "${dlqEnable}" == "true" ]; then
  echo "dlq_target_arn = \"$dlqArn\" " >> ${tfAdditionalVarsFile}
fi

# Adds the lambda role arn
echo "lambda_role_arn = \"$lambdaRoleArn\" " >> ${tfAdditionalVarsFile}

# Each layer ARN is prefixed with LAMBDA_LAYER as environment variables
# Note: parentheses around the while loop are required as while loops in shell run in a sub-process
printenv | grep "lambda-layer" | (while read -r line; do
    echo "Processing lambda layer in env: $line"
    IFS="=" read -ra splitLine <<< "${line}"
    layerArn=${splitLine[1]}
    export layers="${layers} \"${layerArn}\","
done

echo "layers = [${layers::-1}]" >> ${tfAdditionalVarsFile})

terraform_init
terraform_plan
terraform_apply
terraform_output

print_header "Dump output to ConfigMap"

arnConfigMapName=${namingPrefix}-arn
terraform_get arn arnValue
terraform_get invoke_policy invokePolicyValue

cat >/tmp/config.yaml <<EOF
apiVersion: v1
kind: ConfigMap
metadata:
  name: ${arnConfigMapName}
data:
  ARN: ${arnValue}
  InvokePolicy: ${invokePolicyValue}
EOF

run_cmd cat "/tmp/config.yaml"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} apply -f /tmp/config.yaml
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${infraNamespace} apply -f /tmp/config.yaml

echo "Config map ${arnConfigMapName}"
/scripts/wait-until -t 30 -i 5 true kubectl --namespace ${appsNamespace} get cm ${arnConfigMapName} -o yaml

cprint "$0 done "
